################################################################################
# File:             compDeepestOfAll.R
# Created by:       Pierre Lafaye De Micheaux
# First released:   06.11.2018
# Last revised:     06.11.2018
# 
# Contains R-script for computation of the computation of the deepest curve 
# among the deepest curves of each bundle.
# 
# For a detailed description see:
#   Lafaye De Micheaux, P., Mozharovskyi, P. and Vimond, M. (2018). Depth for 
#     curve data and applications.
################################################################################

#setwd("/home/z3458242/MRI")
source("../DTI.R")

# Load necessary R packages
require("curveDepth")
require("AnalyzeFMRI")

curves.subsample <- curveDepth::sample.curves

files <- list.files(path = "../Step2/", pattern = glob2rx("deepest-curve-*.txt"))
deepests <- codes <- rep(NA, length(files))

leftFibers <- as.vector(1:length(deepests), "list")

for (i in 1:length(files)) {
  filename <- files[i]
  codes[i] <- substr(filename, 15, 19)
  deepests[i] <- scan(paste("../Step2/", filename, sep = ""))

  filename <- paste(codes[i], "_1_dwi.voxelcoordsL.txt", sep = "")
  fibers.Left <- f.read.fibers(fibers.file = paste("../Data/", filename, sep = ""))
  fibers.Left.coord <- fibers.Left$fibers.coordinates
  fibers.Left.nb <- fibers.Left$nbfibers
  n <- fibers.Left.nb # Number of curves taken from each side of the brain. It is 1,000 for this specific example

  leftFiberstmp <- list("")
  for (j in 1:n) {
    f1 <- fibers.Left$indices.of.fibers[j, 1]
    f2 <- fibers.Left$indices.of.fibers[j, 2]
    x <- fibers.Left.coord[f1:f2, 1]
    y <- fibers.Left.coord[f1:f2, 2]
    z <- fibers.Left.coord[f1:f2, 3]
    leftFiberstmp[[j]] <- list(voxels = 0, coords = cbind(x, y, z))
  }

  leftFibers[[i]] <- leftFiberstmp[[deepests[i]]]

}

set.seed(1)

search.for.deepest.curves <- function(Fibers = leftFibers, nbtries = 200, k = floor(n ^ 0.8), ptsPerCurve = 256) {
    deepest <- rep(NA, nbtries)
    n <- length(Fibers)
    leftFibersk <- list("")
    for (l in 1:nbtries) {
        subSamp <- sample(1:n, k)
        for (j in 1:k) leftFibersk[[j]] <- Fibers[[subSamp[j]]]
        leftFibersSmplk <- curves.subsample(leftFibersk, ptsPerCurve = ptsPerCurve)
        dssLeft <- depth.curve.Tukey(leftFibersSmplk, leftFibersSmplk, exactEst = FALSE)
        deepest[l] <- subSamp[which.max(dssLeft)]
    }
    return(deepest)
}

time <- system.time({deepest.of.all <- search.for.deepest.curves()})
print(time)
tbl <- table(deepest.of.all)
max.tbl <- max(tbl)
deepest.curve.of.all <- as.numeric(names(which.max(tbl)))
print(deepest.curve.of.all)

print(codes[deepest.curve.of.all])

write(deepests[deepest.curve.of.all], file = paste("deepest-curve-of-all-in-file-", codes[deepest.curve.of.all], ".txt", sep = ""))

save.image(file = paste("deepest-curve-of-all.RData", sep = ""))



